/* crc.h
   crc functions header file
   functions to build and modify crc'd buffs 
   they do not manage buffer length.

   Author : KoreK
   Date : Aug 03, 2004
   License : GPL v2 or above (see COPYING file or get it from the FSF)

   Copyright (c) 2004 KoreK, All Rights Reserved
*/

#ifndef CRC_H
#define CRC_H

/* low-level functions */

/* in those functions, len is always the current length of the buffer */

/* calculate 4 byte crc (mostly internal) */
u_int crc(const u_char *buf,size_t len);

/* check a crc'd buffer, return boolean (0 for failure) */
u_int chk_crc(const u_char *buf,size_t len);

/* add a 4 byte crc to a buffer */
void add_crc(u_char *buf,size_t len);

/* remove last byte (u_char a) of crc'd buffer and correct it */
void trunc_crc(u_char *buf,size_t len,u_char a);
/* undo the removal */
void untrunc_crc(u_char *buf,size_t len,u_char a);

/* append a byte to a crc'd buffer */
void ext_crc1(u_char *buf,size_t len);

/* append a byte to a crc'd buffer */
void ext_crc2(u_char *buf,size_t len);

/* append a 4 byte crc correction to a mask */
void add_crc_mask(u_char *buf,size_t len);


/* higher-level abstraction */


#endif

