/* p80211_const.h
   const for p80211 t

   Author : KoreK
   Date : Aug 26, 2004
   Licence : GPL v2 or above (see accompanying COPYING file)

   Copyright (c) 2004 KoreK, All Rights reserved
*/


#ifndef P80211_C
#error should only be included in p80211.c
#endif

#ifndef P80211_CONST_H
#define P80211_CONST_H

#include "p80211.h"


const char *mac_name[]={
    "DA","SA","BSS","RA","TA"
};

// offset in the header of mac 
// mac_offset[RA_TA_DA_SA][DA]=16 for example
// more likely mac_offset[type_DS[p][ds].macorder][DA]
const size_t mac_offset[8][5]={
    {4,10,16,0,0},  // DA_SA_BSSID
    {4,16,10,0,0},  // DA_BSSID_SA
    {16,10,4,0,0},  // BSSID_SA_DA
    {16,24,0,4,10}, // RA_TA_DA_SA
    {0,0,4,0,10},   // BSSID_TA
    {0,0,10,4,0},   // RA_BSSID
    {0,0,0,4,10},   // RA_TA
    {0,0,0,4,0}     // RA
};



const char *type_name[]={
    "INVALID_FRAME",
    "ASSOC_REQ_FRAME",
    "DATA_FRAME",
    "ASSOC_RESP_FRAME",
    "DATA_CFACK_FRAME",
    "REASSOC_REQ_FRAME",
    "DATA_POLL_FRAME",
    "REASSOC_RESP_FRAME",
    "DATA_CFACK_POLL_FRAME",
    "PROBE_REQ_FRAME",
    "NULL_FRAME",
    "PROBE_RESP_FRAME",
    "NULL_CFACK_FRAME",
    "NULL_POLL_FRAME",
    "NULL_CFACK_POLL_FRAME",
    "BEACON_FRAME",
    "ATIM_FRAME",
    "DISASSOC_FRAME",
    "PS_POLL_FRAME",
    "AUTH_FRAME",
    "RTS_FRAME",
    "DEAUTH_FRAME",
    "CTS_FRAME",
    "ACK_FRAME",
    "CFEND_FRAME",
    "CFEND_ACK_FRAME"
};

const u_int fc_type_mask=h_to_le(~252U);

const u_int fc_type[] = {
    h_to_le(0<<2),  // INVALID_FRAME = 0,
    h_to_le(0<<2),  // ASSOC_REQ_FRAME,
    h_to_le(2<<2),  // DATA_FRAME,
    h_to_le(4<<2),  // ASSOC_RESP_FRAME,
    h_to_le(6<<2),  // DATA_CFACK_FRAME,
    h_to_le(8<<2),  // REASSOC_REQ_FRAME,
    h_to_le(10<<2), // DATA_POLL_FRAME,
    h_to_le(12<<2), // REASSOC_RESP_FRAME,
    h_to_le(14<<2), // DATA_CFACK_POLL_FRAME,
    h_to_le(16<<2), // PROBE_REQ_FRAME,
    h_to_le(18<<2), // NULL_FRAME,
    h_to_le(20<<2), // PROBE_RESP_FRAME,
    h_to_le(22<<2), // NULL_CFACK_FRAME,
    h_to_le(26<<2), // NULL_POLL_FRAME,
    h_to_le(30<<2), // NULL_CFACK_POLL_FRAME,
    h_to_le(32<<2), // BEACON_FRAME,
    h_to_le(36<<2), // ATIM_FRAME,
    h_to_le(40<<2), // DISASSOC_FRAME,
    h_to_le(41<<2), // PS_POLL_FRAME,
    h_to_le(44<<2), // AUTH_FRAME,
    h_to_le(45<<2), // RTS_FRAME,
    h_to_le(48<<2), // DEAUTH_FRAME,
    h_to_le(49<<2), // CTS_FRAME,
    h_to_le(53<<2), // ACK_FRAME,
    h_to_le(57<<2), // CFEND_FRAME,
    h_to_le(63<<2)  // CFEND_ACK_FRAME 
};



#define INVALID_FRAME_1  { .type = INVALID_FRAME,\
                           .header_len = 0,\
                           .mac_order = 0,\
                           .body = 0,\
                           .min_body_len = 0, .max_body_len=0}

#define INVALID_FRAME_GROUP { INVALID_FRAME_1,INVALID_FRAME_1, \
                              INVALID_FRAME_1,INVALID_FRAME_1 }

#define MAX_BODY_LEN_MGT 2312
#define MAX_BODY_LEN_DATA 2312

const struct p80211_frame_struct type_DS[64][4] = {
    // 0000 00 : mgt : Association request
    {{ .type = ASSOC_REQ_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=4, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 0000 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0000 10 : data: Data
    {{ .type = DATA_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a,
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA}},
    // 0000 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 0001 00 : mgt : Association response
    {{ .type = ASSOC_RESP_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=6, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},    
    // 0001 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0001 10 : data: Data + CF-Ack
    {{ .type = DATA_CFACK_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_CFACK_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a,
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_CFACK_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a,
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_CFACK_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA}},
    // 0001 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 0010 00 : mgt : Reassociation request
    {{ .type = REASSOC_REQ_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=10, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 0010 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0010 10 : data: Data + CF-Poll
    {{ .type = DATA_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_POLL_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a,
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a,
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_POLL_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA}},
    // 0010 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 0011 00 : mgt : Reassociation response
    {{ .type = REASSOC_RESP_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=6, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 0011 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0011 10 : data: Data + CF-Ack + CF-Poll
    {{ .type = DATA_CFACK_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_CFACK_POLL_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_CFACK_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA},
     { .type = DATA_CFACK_POLL_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 1,
       .min_body_len = 0, .max_body_len=MAX_BODY_LEN_DATA}},
    // 0011 11 : Reserved
    INVALID_FRAME_GROUP,
    // 0100 00 : mgt : Probe request
    {{ .type = PROBE_REQ_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=2, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 0100 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0100 10 : data: Null Function
    {{ .type = NULL_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0}},
    // 0100 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 0101 00 : mgt : Probe response
    {{ .type = PROBE_RESP_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=12, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 0101 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0101 10 : data: CF-Ack (null)
    {{ .type = NULL_CFACK_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_CFACK_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_CFACK_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_CFACK_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0}},
    // 0101 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 0110 00 : mgt : Reserved
    INVALID_FRAME_GROUP,
    // 0110 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0110 10 : data: CF-Poll (null)
    {{ .type = NULL_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_POLL_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_POLL_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0}},
    // 0110 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 0111 00 : mgt : Reserved
    INVALID_FRAME_GROUP,
    // 0111 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 0111 10 : data: CF-Ack + CF-Poll (null)
    {{ .type = NULL_CFACK_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_CFACK_POLL_FRAME,
       .header_len = 24,
       .mac_order = BSSID_SA_DA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_CFACK_POLL_FRAME,
       .header_len = 24,
       .mac_order = DA_BSSID_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0},
     { .type = NULL_CFACK_POLL_FRAME,
       .header_len = 30,
       .mac_order = RA_TA_DA_SA_a, 
       .body = 0,
       .min_body_len = 0, .max_body_len=0}},
    // 0111 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1000 00 : mgt : Beacon
    {{ .type = BEACON_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=12, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1000 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 1000 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1000 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1001 00 : mgt : ATIM
    {{ .type = ATIM_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1001 01 : ctrl: Reserved
    INVALID_FRAME_GROUP,
    // 1001 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1001 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1010 00 : mgt : Disassociation
    {{ .type = DISASSOC_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=2, .max_body_len=2},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1010 01 : ctrl: PS-Poll
    {{ .type = PS_POLL_FRAME,
       .header_len = 16,
       .mac_order = BSSID_TA_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1010 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1010 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1011 00 : mgt : Authentification
    {{ .type = AUTH_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=6, .max_body_len=MAX_BODY_LEN_MGT},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1011 01 : ctrl: RTS
    {{ .type = RTS_FRAME,
       .header_len = 16,
       .mac_order = RA_TA_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1011 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1011 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1100 00 : mgt : Deauthentification
    {{ .type = DEAUTH_FRAME,
       .header_len = 24,
       .mac_order = DA_SA_BSSID_a,
       .body =1,
       .min_body_len=2, .max_body_len=2},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1100 01 : ctrl: CTS
    {{ .type = CTS_FRAME,
       .header_len = 10,
       .mac_order = RA_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1100 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1100 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1101 00 : mgt : Reserved
    INVALID_FRAME_GROUP,
    // 1101 01 : ctrl: ACK
    {{ .type = ACK_FRAME,
       .header_len = 10,
       .mac_order = RA_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1101 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1101 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1110 00 : mgt : Reserved
    INVALID_FRAME_GROUP,
    // 1110 01 : ctrl: CF-End
    {{ .type = CFEND_FRAME,
       .header_len = 16,
       .mac_order = RA_BSSID_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1110 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1110 11 :       Reserved
    INVALID_FRAME_GROUP,
    // 1111 00 : mgt : Reserved
    INVALID_FRAME_GROUP,
    // 1111 01 : ctrl: CF-End + CF-Ack
    {{ .type = CFEND_ACK_FRAME,
       .header_len = 16,
       .mac_order = RA_BSSID_a,
       .body =0,
       .min_body_len=0, .max_body_len=0},
     INVALID_FRAME_1,
     INVALID_FRAME_1,
     INVALID_FRAME_1},
    // 1111 10 : data: Reserved
    INVALID_FRAME_GROUP,
    // 1111 11 :       Reserved
    INVALID_FRAME_GROUP,
};


#undef INVALID_FRAME_1
#undef INVALID_FRAME_GROUP
#undef MAX_BODY_LEN_MGT
#undef MAX_BODY_LEN_DATA

#endif // P80211_CONST_H


