/* pcap_packet.h
   pcap.h include file 

   Author : KoreK
   Date : Aug 22, 2004
   Licence : GPL v2 or above (see accompanying COPYING file)

   Copyright (c) 2004 KoreK, All Rights Reserved

   Changelog:
   Aug 22, 2004: Initial version
*/

#ifndef PCAP_READWRITE_H
#define PCAP_READWRITE_H

#include <sys/types.h>
#include <pcap.h>


#define ALIGN(a,b) (((a)+(b)-1) & ~((b)-1)) 
#define DEFAULT_ALIGN 4


struct pcap_capture {
    int cap_len;
    int linktype;
    struct pcap_node *first;
    struct pcap_node *last;
    size_t pkt,pkt_read;         // read and unread packets/read packets
};


struct pcap_node {
    struct pcap_pkthdr *pkt_header;
    u_char *pkt_body;
    struct pcap_node *previous;
    struct pcap_node *next;
    struct packet *w_packet;
};


struct packet {
    u_char *frame;
    size_t frame_len;
    struct wep_iv *iv;
    u_char src_mac[6];
    u_char dst_mac[6];
    u_char bssid_mac[6];
};


// Private structure/define
#ifdef PCAP_READWRITE_C 
struct p80211_header {
    size_t header_len;
    int type;
    int fragmented;
    int wep;
    
    const u_char *src_mac;
    const u_char *dst_mac;
    const u_char *bssid_mac;
};

enum {
    p80211_TYPE_MANAGEMENT=0,
    p80211_TYPE_CONTROL=1,
    p80211_TYPE_DATA=2,
    p80211_TYPE_INVALID=3,
    p80211_TYPE_SKIP
};

#endif //PCAP_READWRITE_C



void init_pcap_capture(struct pcap_capture *p);

struct pcap_node *alloc_pcap_node(const struct pcap_pkthdr *pcap_header,
				  const u_char *pcap_body);

void free_pcap_node(struct pcap_node *p);

void free_pcap_capture(struct pcap_capture *p);

int append_pcap_node_capture(struct pcap_node *packet,
			     struct pcap_capture *capture);


/* read/write a pcap file. Returns 0 on success */
int read_pcapfile(const char *name,
		  struct pcap_capture *p);

int write_pcapfile(const char *name,
		   struct pcap_capture *p);

// private functions
#ifdef PCAP_READWRITE_C
void packet_parse_80211(u_char *pcap_capture_list,
			const struct pcap_pkthdr *hdr,
			const u_char *data);

void packet_parse_8023(u_char *pcap_capture_list,
		       const struct pcap_pkthdr *hdr,
		       const u_char *data);

void grab_80211_header(const u_char *data,
		       size_t data_len,
		       struct p80211_header *w_hdr);

#endif //PCAP_READWRITE_C
#endif //PCAP_READWRITE_H
