/* wlan.c
   open/read/write/close a wlan 

   Author : KoreK
   Date : Sep 1, 2004
   Licence : GPL v2 or above (see accompanying COPYING file)

   Copyright (c) 2004 KoreK, All Rights Reserved
*/
#define WLAN_C

#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/poll.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>        // netdevice (7),ioctl 
#include <netinet/in.h>

#include <netpacket/packet.h>
#include <net/ethernet.h>
#include <net/if.h>           // netdevice (7),

#include "p80211.h"
#include "wlan.h"



int open_wlan(char *dev)
{
    int s_fd,err;
    struct ifreq if_req;
    struct sockaddr_ll sa_ll;

    s_fd = socket(PF_PACKET,SOCK_RAW,htons(ETH_P_ALL));

    if (s_fd<0) return -1;

    memset(&if_req,0,sizeof if_req);
    memcpy(if_req.ifr_name,dev,IFNAMSIZ);
    if_req.ifr_name[IFNAMSIZ-1]=0;
    err=ioctl(s_fd,SIOCGIFINDEX,&if_req);
    if (err<0) {
        close(s_fd);
        return -2;
    }
    
    memset(&sa_ll,0,sizeof sa_ll);
    sa_ll.sll_family = AF_PACKET;
    sa_ll.sll_protocol = htons(ETH_P_80211_RAW);  // htons(ETH_P_ALL) ; // 
    sa_ll.sll_ifindex = if_req.ifr_ifindex;
    err=bind(s_fd,(struct sockaddr *)&sa_ll,sizeof sa_ll);
    if (err!=0) {
	fprintf(stderr,"%d\n",if_req.ifr_ifindex);
        close(s_fd);
        return -3;
    }

    return s_fd;
}


int close_wlan(int f_d)
{
    return close(f_d);
}


/* bug/missing case: WDS frame mac4 has to be put in the data section 
   for proper transmission, also the case of some improper frame ? */
/* some control frames, but mgmt/data should be OK */
int write_frame(int fd,
		p80211_frame_t frame)
{
    ssize_t f_len,sz;

    f_len = p80211_data_len(frame) + sizeof(struct p80211_header);
    sz=write(fd,frame,f_len);

    if (sz<0) return -1;
    if (sz<f_len) return -2;
    return sz;
}

extern const char *type_name[];

/* doesn't report error , reset errno before */
p80211_frame_t read_frame(int fd)
{
    char buff[3000];
    ssize_t sz,header_size,body_size;
    p80211_frame_t frame,fr;
    struct p80211_frame_struct fs;
    
    sz = read(fd,buff,3000);
    if ((sz <= 0) || (sz > 3000)) return 0;

    // fprint_hex_buffer(stdout,buff,sz);

    fr = (struct p80211_header *)buff;
    fs = p80211_struct(fr);

    // printf("%s",type_name[fs.type]);

    if (fs.type == INVALID_FRAME) header_size=4;
    else header_size=fs.header_len;

    /* if too small */
    if (header_size > sz) return 0;
    body_size=sz-header_size;

    frame = (p80211_frame_t )malloc(body_size + sizeof(struct p80211_header));
    if (frame == 0) return 0;
    memset(frame,0,sizeof(struct p80211_header));
    memcpy(frame,buff,header_size);
    memcpy(frame->data,buff+header_size,body_size);
    frame->data_len=h_to_le(body_size);
    
    return frame;
}
